/*
 * Copyright (c) KATO Takenori, 1998.
 * 
 * All rights reserved.  Unpublished rights reserved under the copyright
 * laws of Japan.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <fcntl.h>
#include <stdio.h>
#include <osreldate.h>
#include <stdlib.h>
#include <unistd.h>
#include <machine/cpufunc.h>
#include <machine/specialreg.h>

#define	DO_STORE	1
#define	DO_SETS		2
#define	DO_CLEAR	4

void
usage(void)
{
	fprintf(stderr,
	    "usage: cyrixreg -i index          read special register.\n"
	    "       cyrixreg -i index -d data  write special register.\n");
}

int
main(int argc, char *argv[])
{
	int		ch;
	int		fd;
	unsigned char	ccr3;
	unsigned long	idx, data;
	int		automapen = 0;
	int		operation = 0;
	int		mapen = 0;

	if (argc == 1) {
		usage();
		exit(0);
	}

	while ((ch = getopt(argc, argv, "ni:d:s:c:")) != -1) {
		switch (ch) {
		case 'm':
			automapen = 1;
			break;
		case 'i':
			idx = strtoul(optarg, 0, 0);
			break;
		case 'd':
			operation |= DO_STORE;
			data = strtoul(optarg, 0, 0);
			break;
		case 's':
			operation |= DO_SETS;
			data = strtoul(optarg, 0, 0);
			break;
		case 'c':
			operation |= DO_CLEAR;
			data = strtoul(optarg, 0, 0);
			break;
		default:
			usage();
			exit(1);
			/* NOTREACHED */
			break;
		}
	}

	if (idx > 0xff) {
		fprintf(stderr, "Index is out of range: 0x%02x\n", idx);
		exit(1);
	}

	if (operation && data > 0xff) {
		fprintf(stderr, "Data is out of range: 0x%02x\n", data);
		exit(1);
	}

	if ((fd = open("/dev/io", O_RDWR, 0)) == -1) {
		fprintf(stderr, "cyrixreg: Can't access special register.\n");
		exit(1);
	}

	if (automapen && idx < 0xc0 || idx > 0xcf) {
		mapen = 1;
		ccr3 = read_cyrix_reg(CCR3);
		write_cyrix_reg(CCR3, ccr3 | CCR3_MAPEN0);
	}

	switch (operation) {
	case 0:
		data = (unsigned char)read_cyrix_reg(idx);
		printf("%d\n", data);
		break;
	case DO_STORE:
		write_cyrix_reg((unsigned char)idx, (unsigned char)data);
		break;
	case DO_SETS:
		write_cyrix_reg((unsigned char)idx,
						read_cyrix_reg((unsigned char)idx) | data);
	case DO_CLEAR:
		write_cyrix_reg((unsigned char)idx,
						read_cyrix_reg((unsigned char)idx) & ~data);
	default:
		fprintf(stderr, "Data not changed.\n");
		exit(1);
		break;
	}

	if (mapen)
		write_cyrix_reg(CCR3, ccr3);

	close(fd);
	return 0;
}
